/*!
 * grunt-fetch-pages
 * https://github.com/olihel/grunt-fetch-pages.git
 *
 * Copyright (c) 2013 Oliver Hellebusch
 * Released under MIT license (https://raw.github.com/olihel/grunt-fetch-pages/master/LICENSE-MIT)
 */

module.exports = function (grunt) {
  'use strict';

  var fs = require('fs');


  var removeDuplicates = function (array) {
    var i, j;
    for (i = 0; i < array.length; ++i) {
      for (j = i + 1; j < array.length; ++j) {
        if (JSON.stringify(array[i]) === JSON.stringify(array[j])) {
          array.splice(j--, 1);
        }
      }
    }
    return array;
  };

  var createFilePathIfNotExistent = function (file) {
    var fileIndex = file.lastIndexOf('/');
    if (fileIndex !== -1) {
      var path = file.substr(0, fileIndex);
      fs.existsSync(path) || fs.mkdirSync(path);
    }
  };


  var getPagesFromFiles = function (files, baseURL, target) {
    var pages = [];

    files.forEach(function (filePair) {
      var isExpandedPair = filePair.orig.expand || false;

      filePair.src.forEach(function (src) {
        var local;
        var remote;
        if (isExpandedPair) {
          local = src.substr(filePair.orig.cwd.length);
          remote = baseURL + filePair.dest;
        } else {
          local = src;
          remote = baseURL + filePair.dest + src;
        }

        local = target + local;

        pages.push({
          local: local,
          remote: remote
        });
      });
    });

    return pages;
  };

  var getPagesFromURLs = function (urls, target) {
    var pages = [];

    urls.forEach(function (urlObj) {
      var slash = ((target.charAt(target.length - 1) !== '/') && (urlObj.localFile.charAt(urlObj.localFile.length - 1) !== '/')) ? '/' : '';
      var local = target + slash + urlObj.localFile;
      pages.push({
        local: local,
        remote: urlObj.url
      });
    });

    return pages;
  };

  grunt.registerMultiTask('fetchpages', 'Fetch URLs and save the result as local files', function () {
    var request = require('request');

    var done = this.async();
    var options = this.options({
      'target': '',
      'urls': []
    });

    if ((typeof options.filesBaseURL === 'undefined') || (options.filesBaseURL === '')) {
      grunt.log.error('"filesBaseURL" option is mandatory!');
      return false;
    }

    if (options.filesBaseURL.substr(options.filesBaseURL.length - 1) !== '/') {
      options.filesBaseURL += '/';
    }

    if ((options.target !== '') && (options.target.substr(options.target.length - 1) !== '/')) {
      options.target += '/';
    }

    var filesPages = getPagesFromFiles(this.files, options.filesBaseURL, options.target);
    var urlsPages = getPagesFromURLs(options.urls, options.target);
    var pages = removeDuplicates(filesPages.concat(urlsPages));

    var pagesFetched = 0;
    grunt.verbose.writeln('Fetching pages...');
    pages.forEach(function (page) {
      request(page.remote, function (error, response, body) {
        grunt.verbose.writeln('... ' + page.remote + ' -> ' + page.local);
        if (!error && (response.statusCode === 200)) {
          createFilePathIfNotExistent(page.local);
          fs.writeFileSync(page.local, body);
          ++pagesFetched;
          if (pagesFetched === pages.length) {
            done();
          }
        } else {
          response && grunt.log.error('Response error, statusCode=', response.statusCode);
          error && grunt.log.error(error);
          response && response.body && grunt.log.error(response.body);
          done(false);
        }
      });
    });
  });
};
